import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.event.*;

import java.awt.datatransfer.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.*;

/**
* This class represents the main class that drives the GUI program where the user can
* effectively and easily perform image manipulating.
*/
public class Laf extends JFrame{

	JFileChooser fileChooser; // when we want to load/save a file
	JDesktopPane desktop;
	boolean active=true; // is the UI active or not.
	
	// list of internal frames with images currently opened.
	private Vector openImages=new Vector();
	
	// state of all the images, set to the state constants in ImagePanel.
	private int state=0;
   
	Vector clipBoardListeners; // list of clip baord listeners
	
	private static Hashtable cursors=new Hashtable();

 	/** Constructor: Creates the window and menubar and everything.
	 */
	public Laf() {
		super("Laf");
		fileChooser = new JFileChooser();
		fileChooser.setFileFilter(new ImageFileFilter());

		//Make the big window be indented 50 pixels from each edge
		//of the screen.
		int inset = 50;
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setBounds(inset, inset,
                 screenSize.width  - inset*2,
                 screenSize.height - inset*2);

        //Set up the GUI.
		desktop = new JDesktopPane(); //a specialized layered pane
       
       	Dialogs.icApp=this;
		
		// create the desktop, tool bar and menu bar.
		JPanel jp=new JPanel();
		jp.setLayout(new BorderLayout());
		jp.add(createToolBar(), BorderLayout.WEST);
		jp.add(desktop, BorderLayout.CENTER);
		setContentPane(jp);
		setJMenuBar(createMenuBar());

		// create list of clipboard listeners.
		clipBoardListeners=new Vector();
		
		// Listen to the keyboard.
		addKeyListener(KeyBoardManager.getKeyBoardManager());
		desktop.addKeyListener(KeyBoardManager.getKeyBoardManager());
		
		// create singleton cursor objects.
		cursors.put("default", Cursor.getDefaultCursor());
		cursors.put("crosshair", new Cursor(Cursor.CROSSHAIR_CURSOR));
		cursors.put("hand", new Cursor(Cursor.HAND_CURSOR));
		
		// scissor cursor.
		BufferedImage cur=ImageLoader.loadImage("scissor-cursor.gif");
		Dimension d=null;
		if(cur!=null)	
			d=Toolkit.getDefaultToolkit().getBestCursorSize(cur.getWidth(), cur.getHeight());
		if(cur==null || d.width<cur.getWidth() || d.height<cur.getHeight()){
			cursors.put("scissors", cursors.get("crosshair"));
		}else{
			BufferedImage cur2=new BufferedImage(d.width, d.height, BufferedImage.TYPE_INT_ARGB);
			cur2.createGraphics().drawImage(
				cur, (d.width-cur.getWidth())/2, (d.height-cur.getHeight())/2, null);
			Cursor c=Toolkit.getDefaultToolkit().
						createCustomCursor(cur2, new Point(d.width/2, d.height/2), "scissors");
			cursors.put("scissors", c);
		}
		
		// dropper cursor.
		cur=ImageLoader.loadImage("dropper-cursor.gif");
		if(cur!=null)	
			d=Toolkit.getDefaultToolkit().getBestCursorSize(cur.getWidth(), cur.getHeight());
		if(cur==null || d.width<cur.getWidth() || d.height<cur.getHeight()){
			cursors.put("dropper", cursors.get("default"));
		}else{
			BufferedImage cur2=new BufferedImage(d.width, d.height, BufferedImage.TYPE_INT_ARGB);
			cur2.createGraphics().drawImage(
				cur, (d.width-cur.getWidth())/2, (d.height-cur.getHeight())/2, null);
			Cursor c=Toolkit.getDefaultToolkit().
						createCustomCursor(cur2,
							new Point((d.width-cur.getWidth())/2,
								(d.height-cur.getHeight())/2+cur.getHeight()), "dropper");
			cursors.put("dropper", c);
		}
       //Make dragging a little faster but perhaps uglier.
       //desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
	}

	/** Creates a menu bar for this application.
	 * @return Menubar for this application.
	 */
	private JMenuBar createMenuBar() {
		JMenuBar menuBar = new JMenuBar();
		menuBar.setFocusable(false);
		JMenu menu;
		JMenuItem menuItem;

		menu = new JMenu("File");
		//a group of JMenuItems
		
		// menu item for creating a new image.
		menuItem = new JMenuItem("New Image");
		ActionListener as=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!active) return;
				int w=0, h=0;
				BufferedImage cb;
				if((cb=getClipboard())!=null){
					w=cb.getWidth();
					h=cb.getHeight();
				}
				setActive(false);
				Dimension d=Dialogs.showSizeInputDialog(w, h);
				setActive(true);
				if(d!=null)
					createInternalFrame("Untitled", new ImagePanel(
						new BufferedImage(d.width, d.height, BufferedImage.TYPE_INT_ARGB)), null, true);
			}
		};
		menuItem.addActionListener(as);
		KeyBoardManager.getKeyBoardManager().addHitListener("Ctrl n", as);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(new Character('O'), ActionEvent.CTRL_MASK));
		menu.add(menuItem);
		
		// menu item for opening any image.
		menuItem = new JMenuItem("Open Image");
		as=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!active) return;
				//goes through the process of selecting the file and using as the image
				//with its corresponding image panel
				setActive(false);
				int returnValue =fileChooser.showOpenDialog(Laf.this);
				setActive(true);
				if (returnValue == JFileChooser.APPROVE_OPTION) {
					String fileName =fileChooser.getSelectedFile().getAbsolutePath();

					// source image.
					createInternalFrame(fileName, new ImagePanel(fileName), null, true);
				}
			}
		};
		menuItem.addActionListener(as);
		KeyBoardManager.getKeyBoardManager().addHitListener("Ctrl o", as);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(new Character('O'), ActionEvent.CTRL_MASK));
		menu.add(menuItem);
		
		menu.addSeparator();
		
		// menu item for saving an image.
		menuItem = new JMenuItem("Save as");
		as=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// get the selected frame.
				if(!active || !(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame icif=(ICInternalFrame)desktop.getSelectedFrame();
				if(icif==null)
					return;
				
				// get the image for selection.
				BufferedImage im=((ImagePanel) icif.getContentPane()).getImage();
				if(im==null)
					return;

				setActive(false);
				fileChooser.setApproveButtonText("Save");
				setActive(true);
				//goes through the process of selecting the file and using as the image with its corresponding image panel
				int returnValue =fileChooser.showOpenDialog(Laf.this);
				fileChooser.setApproveButtonText("Open");
				if (returnValue == JFileChooser.APPROVE_OPTION) {
					String fileName =fileChooser.getSelectedFile().getAbsolutePath();
					File f=new File(fileName);
					// check if filename extension is correct.
					if(!f.getName().toLowerCase().matches(".*\\.png$")){
						JOptionPane.showMessageDialog(desktop,
							"Only the png file type is supported.");
						return;
					}else if(f.getName().lastIndexOf(".")==0){
						JOptionPane.showMessageDialog(desktop,
							"You have to specify a file name as in blah.png.");
						return;
					}
					// check if the file exists.
					if(f.length()!=0){
						int opt=JOptionPane.showConfirmDialog(desktop,
							"A file exists with this name. Do you want to overwrite it?");
						if(opt!=JOptionPane.YES_OPTION)
							return;
					}
					
					try{
						// write to the file.
						ImageIO.write(im, "png", f);
					}catch(IOException ex){
						JOptionPane.showMessageDialog(desktop, "Couldn't save the file.");
					}
				}
			}
		};
		menuItem.addActionListener(as);
		KeyBoardManager.getKeyBoardManager().addHitListener("Ctrl s", as);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(new Character('S'), ActionEvent.CTRL_MASK));
		menu.add(menuItem);
		
		menu.addSeparator();

		// Menu item for quitting.
		menuItem = new JMenuItem("Quit");
		as=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!active) return;
				System.exit(0);
			}
		};
		menuItem.addActionListener(as);
		KeyBoardManager.getKeyBoardManager().addHitListener("Ctrl q", as);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(new Character('Q'), ActionEvent.CTRL_MASK));

		menu.add(menuItem);

		menuBar.add(menu);
		
		// menu for image editing options.
		menu=new JMenu("Edit");
		menuBar.add(menu);
		
		// menu item for undoing/redoing operations.
		menuItem=new JMenuItem("Undo/Redo");
		menu.add(menuItem);
		as=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// get the selected frame.
				if(!active || !(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame icif=(ICInternalFrame)desktop.getSelectedFrame();
				if(icif==null)
					return;
					
				// get the image for selection.
				((ImagePanel) icif.getContentPane()).setPrevState();
			}
		};
		KeyBoardManager.getKeyBoardManager().addHitListener("Ctrl z", as);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(new Character('Z'), ActionEvent.CTRL_MASK));
		menuItem.addActionListener(as);
		
		// menu item for copying part of an image.
		menuItem=new JMenuItem("Copy");
		menu.add(menuItem);
		as=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// get the selected frame.
				if(!active || !(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame icif=(ICInternalFrame)desktop.getSelectedFrame();
				if(icif==null)
					return;
					
				// get the image for selection.
				BufferedImage s=((ImagePanel) icif.getContentPane()).getSelection();
				if(s==null){
					// nothing was selected in the image.
					javax.swing.SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							JOptionPane.showMessageDialog(desktop,"Nothing is selected! Use the selection tool to select an area to copy.");
						}
					});
				}else
					setClipboard(s);
			}
		};
		KeyBoardManager.getKeyBoardManager().addHitListener("Ctrl c", as);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(new Character('C'), ActionEvent.CTRL_MASK));
		menuItem.addActionListener(as);
		
		// menu item for pasting clipboard into an image.
		menuItem=new JMenuItem("Paste");
		menu.add(menuItem);
		as=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// get the selected fram.
				if(!active || !(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame icif=(ICInternalFrame)desktop.getSelectedFrame();
				if(icif==null)
					return;
						
				BufferedImage s=getClipboard();
				if(s==null){
					// nothing was copied prior.
					javax.swing.SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							JOptionPane.showMessageDialog(desktop,
								"Nothing to paste! First copy some (part of an) image.");
						}
					});
				}else{
					ImagePanel ip=((ImagePanel) icif.getContentPane());
					ip.pasteImage(s);
				}
						
			}
		};
		KeyBoardManager.getKeyBoardManager().addHitListener("Ctrl v", as);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(new Character('V'), ActionEvent.CTRL_MASK));
		menuItem.addActionListener(as);
		
		// menu item for extraction foreground from an image.
		menuItem=new JMenuItem("Copy foreground");
		menu.add(menuItem);
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// get the selected frame.
				if(!(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame icif=(ICInternalFrame)desktop.getSelectedFrame();
				if(icif==null)
					return;
				
				// get the image for selection.
				BufferedImage im=((ImagePanel) icif.getContentPane()).getSelection();
				if(im==null){
					im=((ImagePanel) icif.getContentPane()).getImage();
					if(im==null) return;
				}
				setActive(false);
				BufferedImage fg=Dialogs.showForegroundExtr(im);
				if(fg!=null){
					setClipboard(fg);
				}
				setActive(true);
			}
		});
		
		menu.addSeparator();
		
		JMenu menu2=new JMenu("Transform");
		menu.add(menu2);

		//warping menu item.
		menuItem = new JMenuItem("Warp Image");
		menu2.add(menuItem);

		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame f=(ICInternalFrame)desktop.getSelectedFrame();
				if(f==null)
					return;
				
				ImagePanel ip=((ImagePanel)f.getContentPane());
				BufferedImage sel=ip.getSelection(), im=ip.getImage();
				if(im!=null){
					if(sel!=null){
						for(int y=0; y<sel.getHeight(); y++)
							for(int x=0; x<sel.getWidth(); x++)
								if(sel.getRGB(x, y)!=0)
									im.setRGB(ip.getSelectionX()+x, ip.getSelectionY()+y, 0);
					}else{
						sel=im;
						im=new BufferedImage(sel.getWidth(), sel.getHeight(), BufferedImage.TYPE_INT_ARGB);
					}
					f.setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
					f.setContentPane(new WarpTargetPanel(sel, im, ip.getSelectionX(), ip.getSelectionY(),
						Laf.this, f));
					setActive(false);
				}
			}
		});
		

		// Tools menu
		menu = new JMenu("Tools");
		menuBar.add(menu);
		
		// menu item for opening an image to start mosaicing.
		menuItem = new JMenuItem("Mosaic Image");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame f=(ICInternalFrame)desktop.getSelectedFrame();
				if(f==null)
					return;
				
				ImagePanel ip=((ImagePanel)f.getContentPane());
				BufferedImage im;
				if((im=ip.getSelection())!=null || (im=ip.copyImage())!=null){
					//setActive(false);
					im=Dialogs.showMosaicDialog(im);
					if(im!=null){
						ip=new ImagePanel(im);
						createInternalFrame("Mosaic result", ip, null, true);
					}
					//setActive(true);
				}
			}
		});
		menu.add(menuItem);

		//greyscaling menu item.
		menuItem = new JMenuItem("Greyscale Image");
		menu.add(menuItem);

		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame f=(ICInternalFrame)desktop.getSelectedFrame();
				if(f==null)
					return;
					
				ImagePanel ip=((ImagePanel)f.getContentPane());
				BufferedImage im;
				setActive(false);
				if((im=ip.getSelection())!=null){
					ip.paintImage(Paint.convertGreyscale(im), ip.getSelectionX(), ip.getSelectionY());
				}else if((im=ip.getImage())!=null){
					ip.setImage(Paint.convertGreyscale(im));
				}
				setActive(true);
			}
		});
		
		// Image Seperation menu item
		menuItem = new JMenuItem("Image Channel Seperation");
		menu.add(menuItem);

		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame f=(ICInternalFrame)desktop.getSelectedFrame();
				if(f==null)
					return;
	
				ImagePanel ip=((ImagePanel)f.getContentPane());
				BufferedImage im;
				setActive(false);
				if((im=ip.getSelection())!=null || (im=ip.getImage())!=null){
					ip =new ImagePanel(Paint.imageSeperation(im, "red"));
					createInternalFrame("Image Channel Seperation: Red", ip, null, true);
					ip = new ImagePanel(Paint.imageSeperation(im, "green"));
					createInternalFrame("Image Channel Seperation: Green", ip, null, true);
					ip = new ImagePanel(Paint.imageSeperation(im, "blue"));
					createInternalFrame("Image Channel Seperation: Blue", ip, null, true);
				}
				setActive(true);
			}
		});
		
		// Gaussian smoothing menu item
		menuItem = new JMenuItem("Gaussian Smoothing");
		menu.add(menuItem);

		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame f=(ICInternalFrame)desktop.getSelectedFrame();
				if(f==null)
					return;

				setActive(false);
				ImagePanel ip=((ImagePanel)f.getContentPane());
				BufferedImage im;
				if((im=ip.getSelection())!=null){
					BufferedImage gaussian =Paint.getGaussian(desktop,im, -1);
					setActive(true);
					if(gaussian==null) return;
					ip.paintImage(gaussian, ip.getSelectionX(), ip.getSelectionY());
				}else if((im=ip.getImage())!=null){
					BufferedImage gaussian =Paint.getGaussian(desktop,im, -1);
					setActive(true);
					if(gaussian==null) return;
					ip.setImage(gaussian);
				}
			}
		});

		// LoG producing menu item.
		menuItem = new JMenuItem("Laplacian of Gaussian");
		menu.add(menuItem);

		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame f=(ICInternalFrame)desktop.getSelectedFrame();
				if(f==null)
					return;
					
				ImagePanel ip=((ImagePanel)f.getContentPane());
				BufferedImage im=ip.getSelection();
				boolean select=true;
				if(im==null && (im=ip.getImage())!=null){
					select=false;
				}else if(im==null)
					return;

				setActive(false);
				double[][] log = Paint.getLoG(desktop, im, -1);
				setActive(true);
						
				if(log==null) return;
				BufferedImage logimg =
					new BufferedImage(
						log[0].length,log.length,BufferedImage.TYPE_INT_ARGB);

				double max = 0;
				int tmp;
				// find maximum absolute value.
				for (int i = 0; i < log.length; i++) {
					for (int j = 0; j < log[0].length; j++) {
						if (Math.abs(log[i][j]) > max)
							max = Math.abs(log[i][j]);
					}
				}
				// normalize and shift by 128 for the image.
				for (int i = 0; i < log.length; i++) {
					for (int j = 0; j < log[0].length; j++) {
						tmp = (int) (log[i][j] / max * 127 + 128);
						logimg.setRGB(j, i, 0xff000000+(tmp << 16) + (tmp << 8) + tmp);
					}
				}

				if(select){
					ip.pasteImage(logimg);
				}else{
					ip.setImage(logimg);
				}
			}
		});

		// zero crossings producing menu item.
		menuItem = new JMenuItem("Edges");
		menu.add(menuItem);

		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame f=(ICInternalFrame)desktop.getSelectedFrame();
				if(f==null)
					return;
	
				ImagePanel ip=((ImagePanel)f.getContentPane());
				BufferedImage im=ip.getSelection();
				boolean select=true;
				if(im==null && (im=ip.getImage())!=null){
					select=false;
				}else if(im==null)
					return;

				setActive(false);
				int[][] zx = Paint.getZeroXings(desktop, im, -1);
				setActive(true);
				if(zx==null)
					return;
					
				BufferedImage zximg =
					new BufferedImage(
						im.getWidth(),
						im.getHeight(),
						BufferedImage.TYPE_INT_ARGB);
				// set the values in the image.
				for (int i = 0; i < zx.length; i++) {
					for (int j = 0; j < zx[0].length; j++) {
						zximg.setRGB(j, i, 0xff000000+
							(zx[i][j] << 16) + (zx[i][j] << 8) + zx[i][j]);
					}
				}

				if(select){
					ip.pasteImage(zximg);
				}else{
					ip.setImage(zximg);
				}
			}
		});
		
		//image gradient producing menu item.
		menuItem = new JMenuItem("Image Gradient");
		menu.add(menuItem);

		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(!(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame f=(ICInternalFrame)desktop.getSelectedFrame();
				if(f==null)
					return;
					
				BufferedImage im=((ImagePanel)f.getContentPane()).getImage();
				if(im==null)
					return;

				setActive(false);
				GradVector[][] grad =Paint.getGradient(im);
				setActive(true);
				
				BufferedImage gimg=new BufferedImage(grad[0].length, grad.length, BufferedImage.TYPE_INT_ARGB);
				
				// get maximum horizontal component.
				double max=0;
				for(int i=0; i<grad.length; i++){
					for(int j=0; j<grad[0].length; j++){
						if(Math.abs(grad[i][j].hor*grad[i][j].mag)>max)
							max=Math.abs(grad[i][j].hor*grad[i][j].mag);
					}
				}
				
				// normalize and shift by 128 for the image.
				int tmp;
				for(int i=0; i<grad.length; i++){
					for(int j=0; j<grad[0].length; j++){
						tmp=(int)(grad[i][j].hor*grad[i][j].mag/max*127)+128;
						gimg.setRGB(j, i, 0xff000000+(tmp<<16)+(tmp<<8)+tmp);
					}
				}
				
				createInternalFrame(
					"Horizontal Gradient Components",
					new ImagePanel(gimg), null, true);
					
				gimg=new BufferedImage(grad[0].length, grad.length, BufferedImage.TYPE_INT_ARGB);
				
				// get maximum vertical component.
				max=0;
				for(int i=0; i<grad.length; i++){
					for(int j=0; j<grad[0].length; j++){
						if(Math.abs(grad[i][j].vert*grad[i][j].mag)>max)
							max=Math.abs(grad[i][j].vert*grad[i][j].mag);
					}
				}
				
				// normalize and shift by 128 for the image.
				for(int i=0; i<grad.length; i++){
					for(int j=0; j<grad[0].length; j++){
						tmp=(int)(grad[i][j].vert*grad[i][j].mag/max*127)+128;
						gimg.setRGB(j, i, 0xff000000+(tmp<<16)+(tmp<<8)+tmp);
					}
				}
				
				createInternalFrame(
					"Vertical Gradient Components",
					new ImagePanel(gimg),null,true);
			}
		});

		//gradient magnitude producing menu item.
		menuItem = new JMenuItem("Image Gradient Magnitude");
		menu.add(menuItem);

		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				if(!(desktop.getSelectedFrame() instanceof ICInternalFrame))
					return;
				ICInternalFrame f=(ICInternalFrame)desktop.getSelectedFrame();
				if(f==null)
					return;

				ImagePanel ip=((ImagePanel)f.getContentPane());
				BufferedImage im=ip.getSelection();
				boolean select=true;
				if(im==null && (im=ip.getImage())!=null){
					select=false;
				}else if(im==null)
					return;

				setActive(false);
				GradVector[][] grad =Paint.getGradient(im);
				setActive(true);
				BufferedImage gimg =
					new BufferedImage(
						grad[0].length,
						grad.length,
						BufferedImage.TYPE_INT_ARGB);
				// find the maximum magnitude for normalizing.
				double max = 0;
				for (int i = 0; i < grad.length; i++) {
					for (int j = 0; j < grad[0].length; j++) {
						if (grad[i][j].mag > max)
							max = grad[i][j].mag;
					}
				}

				int tmp;
				// fill the image with normalized gradient magintudes.
				for (int i = 0; i < grad.length; i++) {
					for (int j = 0; j < grad[0].length; j++) {
						tmp = (int) (grad[i][j].mag / max * 255);
						gimg.setRGB(j, i, 0xff000000+(tmp << 16) + (tmp << 8) + tmp);
					}
				}

				if(select){
					ip.pasteImage(gimg);
				}else{
					ip.setImage(gimg);
				}
			}
		});
		
		menu=new JMenu("Help");
		menuBar.add(menu);
		
		// help contents.
		menuItem=new JMenuItem("Contents");
		menu.add(menuItem);
		as=new ActionListener(){
			public void actionPerformed(ActionEvent e){
				Dialogs.showHelp();
			}
		};
		menuItem.addActionListener(as);
		KeyBoardManager.getKeyBoardManager().addHitListener("F1", as);
		menuItem.setAccelerator(KeyStroke.getKeyStroke("F1"));
		
		menu.addSeparator();
		
		// about ImageCompositiing
		menuItem=new JMenuItem("About Laf");
		menu.add(menuItem);
		as=new ActionListener(){
			public void actionPerformed(ActionEvent e){
				Dialogs.showAboutDialog();
			}
		};
		menuItem.addActionListener(as);
		

		return menuBar;
	}
	
	/**
	 * Creates the toolbar for the program.
	 * @return Tool bar for the program.
	 */
	private JToolBar createToolBar(){
		JToolBar tb=new JToolBar(JToolBar.VERTICAL);
		tb.setFocusable(false);
		
		// button for the regular pointer tool. Sets the opened image panels to default state.
		JButton b=new JButton();
		b.setToolTipText("Pointer tool");
		b.setFocusable(false);
		b.setIcon(new ImageIcon("tooltip.png"));
		tb.add(b);
		
		b.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				ImagePanel ip;
				if(!active){
					for(int i=0; i<openImages.size(); i++){
						ip=(ImagePanel)((ICInternalFrame)openImages.get(i)).getContentPane();
						if(ip.getState()==ImagePanel.MISC){
							if(!ip.finilizeOperation())
								return;
						}
					}
				}
				state=ImagePanel.POINTER;
				for(int i=0; i<openImages.size(); i++){
					ip=(ImagePanel)((ICInternalFrame)openImages.get(i)).getContentPane();
					ip.setState(ImagePanel.POINTER);
				}
			}
		});
		
		// button using the selection tool. Sets the opened image panels to SELECT state.
		b=new JButton();
		b.setToolTipText("Selection tool");
		b.setFocusable(false);
		b.setIcon(new ImageIcon("crop.png"));
		tb.add(b);
		b.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				ImagePanel ip;
				if(!active){
					for(int i=0; i<openImages.size(); i++){
						ip=(ImagePanel)((ICInternalFrame)openImages.get(i)).getContentPane();
						if(ip.getState()==ImagePanel.MISC){
							if(!ip.finilizeOperation())
								return;
						}
					}
				}
				state=ImagePanel.SELECT;
				for(int i=0; i<openImages.size(); i++){
					ip=(ImagePanel)((ICInternalFrame)openImages.get(i)).getContentPane();
					ip.setState(ImagePanel.SELECT);
				}
			}
		});
		
		// button using the scissor tool. Sets the opened image panels to SCISSOR state.
		b=new JButton();
		b.setToolTipText("Scissoring tool");
		b.setFocusable(false);
		b.setIcon(new ImageIcon("scissors.png"));
		tb.add(b);
		b.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				ImagePanel ip;
				if(!active){
					for(int i=0; i<openImages.size(); i++){
						ip=(ImagePanel)((ICInternalFrame)openImages.get(i)).getContentPane();
						if(ip.getState()==ImagePanel.MISC){
							if(!ip.finilizeOperation())
								return;
						}
					}
				}
				state=ImagePanel.SCISSOR;
				for(int i=0; i<openImages.size(); i++){
					ip=(ImagePanel)((ICInternalFrame)openImages.get(i)).getContentPane();
					ip.setState(ImagePanel.SCISSOR);
				}
			}
		});
		
		return tb;
	}

    /**
	 * Creates an internal frame within the desktop.
	 * @param frameTitle Title of the frame.
	 * @param ip image panel.
	 * @param previousInternalFrame Last frame to replace with this one.
	 * @param closable Closable or not.
	 * @return The created frame.
	 */
	public ICInternalFrame createInternalFrame(
		String frameTitle,
		ImagePanel ip,
		ICInternalFrame previousInternalFrame,
		boolean closable) {
		if(ip.getImage()==null)
			return previousInternalFrame;
			
		ICInternalFrame icif = null;

		//replaces the old internal frame if there is one
		if (previousInternalFrame != null) {
			desktop.remove(previousInternalFrame);
			openImages.remove(previousInternalFrame);
			previousInternalFrame.dispose();
		}
		icif = new ICInternalFrame(frameTitle, ip);
		icif.addKeyListener(KeyBoardManager.getKeyBoardManager());
		icif.setVisible(true); //necessary as of 1.3
		desktop.add(icif);
		openImages.add(icif);
		ip.setState(state);
		
		try {
			icif.setSelected(true);
		} catch (java.beans.PropertyVetoException e) {
		}

		if (closable) {
			icif.setClosable(true);
			icif.addInternalFrameListener(new InternalFrameAdapter() {
				public void internalFrameClosed(InternalFrameEvent e) {
					desktop.remove(e.getInternalFrame());
					openImages.remove(e.getInternalFrame());
					((ImagePanel)e.getInternalFrame().getContentPane()).cleanUp();
					desktop.grabFocus();
				}
			});
		}
		return icif;
	}

	/**
	 * Create the GUI and show it.  For thread safety,
	 * this method should be invoked from the
	 * event-dispatching thread.
	 */
	private static void createAndShowGUI() {
		//Make sure we have nice window decorations.

		//Create and set up the window.
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);
		Laf frame = new Laf();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		//Display the window.
		frame.setVisible(true);
	}

	/** Set the clipboard image.
	 * @param cbimg Image to save to the clipboard.
	 */
	public void setClipboard(BufferedImage cbimg) {
		Toolkit.getDefaultToolkit().getSystemClipboard().setContents(new ImageSelection(cbimg), null);
		fireClipBoardEvent(cbimg);
	}
	
	/** Returns the clipboard image.
	 * @return Clipboard image.
	 */
	public BufferedImage getClipboard(){
		Transferable t = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(null);
    
		try {
			if (t != null && t.isDataFlavorSupported(DataFlavor.imageFlavor)) {
				Image img = (Image)t.getTransferData(DataFlavor.imageFlavor);
				BufferedImage bimg=new BufferedImage(img.getWidth(null),
					img.getHeight(null), BufferedImage.TYPE_INT_ARGB);
				bimg.createGraphics().drawImage(img, 0, 0, null);
				return bimg;
			}
		} catch (UnsupportedFlavorException e) {
		} catch (IOException e) {
		}
		return null;
	}
	
	/** Adds a clip board listener object.
	 * @param cb Listener.
	 */
	public void addClipBoardListener(ClipBoardListener cb){
		clipBoardListeners.add(cb);
	}
	
	/** Removes a clip board listener object.
	 * @param cb Listener to remove.
	 */
	public void removeClipBoardListener(ClipBoardListener cb){
		clipBoardListeners.remove(cb);
	}
	
	/** Sends notifications to all clip board listeners.
	 */
	public void fireClipBoardEvent(BufferedImage cbimg){
		for(int i=0; i<clipBoardListeners.size(); i++)
			((ClipBoardListener)clipBoardListeners.elementAt(i)).clipBoardChanged(cbimg);
	}
	
	/**
	 * Sets active state of the UI.
	 * @param active true active, false not.
	 */
	public void setActive(boolean active){
		this.active=active;
		Component [] comps=getJMenuBar().getComponents();
		for(int i=0; i<comps.length; i++)
			((JComponent)comps[i]).setEnabled(active);
	}
	
	/**
	 * Returns the state of the image panels holding opened images.
	 * @return State of image panels.
	 */
	public int getImageState(){
		return state;
	}
	
	/**
	 * Returns cursor object.
	 * @param type Name of the cursor.
	 * @return Cursor object.
	 */
	public static Cursor getCursor(String type){
		return (Cursor)cursors.get(type);
	}

	public static void main(String[] args) {

		//Schedule a job for the event-dispatching thread:
		//creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});
	}
}
