/**
 * Class for lumping several progressable operations together under a single progress bar.
 */
public abstract class ProcessesLump{
	private Object result;
	
	/**
	 * Runs the processes lump with the progress bar shown.
	 * @return rsult of the processes.
	 */
	public Object runOperations(){
		if(!precompute())
			return null;
		
		// set initial progress.
		Paint.getProgress().getSetCurrentProgress(1);
		// total progress.
		int total=estimateProgress();
		// separate thread to run the processes.
		Thread t=new Thread(new Runnable(){
			public void run(){
				result=exec();
				// reset the progress.
				Paint.getProgress().getSetCurrentProgress(0);
			}
		});
		t.start();
		// show progress bar.
		Dialogs.showProgressBar(Paint.getProgress(), total);
		return result;
	}

	/**
	 * Does initial precomputations.
	 * @return true to continue to run processes, false otherwise.
	 */
	protected abstract boolean precompute();
	
	/**
	 * Estimate total progress for all the processes.
	 * @return Estimate of the total progress.
	 */
	protected abstract int estimateProgress();
	
	/**
	 * Execute the processes.
	 * @return Result from the processes.
	 */
	protected abstract Object exec();
}
